"""Class to represent a departure from the OGT rest api."""


class OGTDeparture(object):
    """Class to store information about a departure."""

    def __init__(self, name='', date_time='', towards='', stop_point=''):
        """Initialize an OGTDeparture."""
        self._name = name
        self._date_time = date_time
        self._towards = towards
        self._stop_point = stop_point

    def get_name(self):
        """Return the name of the departure."""
        return self._name

    def get_date_time(self):
        """Return the date/time of the departure."""
        return self._date_time

    def get_towards(self):
        """Return the towards of the departure."""
        return self._towards

    def get_stop_point(self):
        """Return the stop point of the departure."""
        return self._stop_point



"""OGTRest exposes the OGT rest api."""
import requests
from .ogtstop import OGTStop
from .ogtdeparture import OGTDeparture


class OGTRest(object):
    """Static class that exposes the OGT rest api."""

    URL_REST = 'https://rest.ostgotatrafiken.se/'
    METHOD_STOPS_FIND = 'stops/Find'
    METHOD_STOPS_INFOS = 'stops/Infos'
    METHOD_STOP_DEPARTURES = 'stopdepartures/departures'

    @staticmethod
    def get(method, params):
        """Perform a http get requests to the OGT rest api."""
        url = OGTRest.URL_REST + method
        response = requests.get(url, params=params)
        if response.status_code == 200:
            return response.json()
        return None

    @staticmethod
    def stops_find(query):
        """Find stops that matches the query."""
        # Define method and params
        method = OGTRest.METHOD_STOPS_FIND
        params = {
            'q': query,
            'pointType': 'stop',
        }

        # Send request
        stops = OGTRest.get(method, params)

        # Convert to list of OGTStop
        ogt_stops = []
        if stops is not None:
            for stop in stops:
                name = stop['OgtStopUrlSegment']
                id_ = stop['Id']
                pretty_name = stop['PlaceName']
                ogt_stops.append(OGTStop(name, id_, pretty_name))

        # Return stops
        return ogt_stops

    @staticmethod
    def stops_infos(ids):
        """Find stops that matches the ids."""
        # Define method and params
        method = OGTRest.METHOD_STOPS_INFOS
        if isinstance(ids, (list,)):
            ids = ','.join(map(str, ids))
        params = {
            'ids': ids,
        }

        # Send request
        stops = OGTRest.get(method, params)

        # Convert to list of OGTStop
        ogt_stops = []
        if stops is not None:
            for stop in stops:
                name = stop['OgtStopUrlSegment']
                id_ = stop['Id']
                pretty_name = stop['PlaceName']
                ogt_stops.append(OGTStop(name, id_, pretty_name))

        # Return stops
        return ogt_stops

    @staticmethod
    def stop_departures(stop_id, num_departures=5):
        """Find stop departures that matches the stop_id."""
        # Define method and params
        method = OGTRest.METHOD_STOP_DEPARTURES
        sort_order = 'DepartureTime'  # LineNumber
        params = {
            'stopAreaId': stop_id,
            'date': '',
            'delay': 0,
            'maxNumberOfResultPerColumn': num_departures,
            'columnsPerPageCount': 1,
            'pagesCount': 1,
            'lines': ','.join([]),
            'trafficTypes': ','.join([]),
            'stopPoints': ','.join([]),
            'sortOrder': sort_order,
            'useDaySeparator': False,
        }

        # Send request
        data_departures = OGTRest.get(method, params)['groups'][0]

        # Convert to list of OGTDeparture
        ogt_departures = []
        if data_departures is not None:
            for data_departure in data_departures:
                data_line = data_departure['Line']
                name = data_line['Ogt']['OgtLineUrlSegment']
                date_time = data_line['JourneyDateTime']
                towards = OGTRest.stops_infos(
                    data_line['Ogt']['PassingStops'][-1])[0]
                stop_point = data_line['StopPointAliasFormatted']
                ogt_departures.append(
                    OGTDeparture(name, date_time, towards, stop_point))

        # Return departures
        return ogt_departures



"""Class to represent a stop from the OGT rest api."""


class OGTStop(object):
    """Class to store information about a stop."""

    def __init__(self, name='', id_=-1, pretty_name=''):
        """Initialize an OGTStop."""
        self._name = name
        self._id = id_
        self._pretty_name = pretty_name

    def get_name(self):
        """Return the name of the stop."""
        return self._name

    def get_id(self):
        """Return the id of the stop."""
        return self._id

    def get_pretty_name(self):
        """Return the pretty name of the stop."""
        return self._pretty_name



"""Init file for pyogt."""
from .ogtrest import OGTRest
from .ogtstop import OGTStop


def find_stop(name):
    """Find a stop by name."""
    stops = OGTRest.stops_find(query=name)

    for stop in stops:
        if stop.get_name() == name:
            return stop

    return OGTStop()


def find_stop_by_id(id_):
    """Find a stop by id."""
    stops = OGTRest.stops_infos(ids=id_)

    for stop in stops:
        if stop.get_id() == id_:
            return stop

    return OGTStop()


def get_departures(name):
    """Get departures from stop."""
    stop = find_stop(name)
    departures = OGTRest.stop_departures(stop.get_id())
    return departures



"""Setup script for pyogt."""
from setuptools import setup, find_packages

setup(
    name='pyogt',
    version='0.1.0',
    author='Claes Hallstrom',
    author_email='hallstrom.claes@gmail.com',
    description='Retrieve departure information from OstgotaTrafiken',
    license='Apache License 2.0',
    url='https://github.com/claha/pyogt',
    packages=find_packages(exclude=['tests', 'tests.*']),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
    ],
)
